{% load i18n static %}<!DOCTYPE html>
<html>
  <head><title>{% trans 'Popup closing...' %}</title></head>
  <body>
    <script type="text/javascript">
        window.close();
    </script>
  </body>
</html>
