{% load menu_tags %}


{% for child in children %}
	<a class="list-group-item{% if child.selected %} active{% endif %}{% if child.ancestor %} ancestor{% endif %}{% if child.sibling %} sibling{% endif %}{% if child.descendant %} descendant{% endif %}" href="{{ child.attr.redirect_url|default:child.get_absolute_url }}">{{ child.get_menu_title }}</a>
	{% if child.children %}
    <div class="list-group">
        {% show_menu from_level to_level extra_inactive extra_active template "" "" child %}
    </div>
	{% endif %}
{% endfor %}
