{% load i18n %}
<form class="form-horizontal" method="get" id="list_filter_form">
    <div class="form-group">
        {% for item in form %}
        <div class="col-sm-6">
            <label class="control-label col-md-4" for="{{ item.id_for_label }}">
                {{ item.label }}
            </label>
            <div class="col-md-8{% if item.errors %} alert alert-danger{% endif %}">
                {{ item }}
            </div>
        </div>
        {% endfor %}
        <div class="col-xs-12 right">
            <button class="btn btn-primary" type="submit">{% trans 'search' %}</button>
            <a class="btn btn-default" href="{{ request.path }}">{% trans 'cancel search' %}</a>
        </div>
    </div>
</form>
